use chrono::Duration;
use chrono_tz::Tz;
use cron::Schedule;
use serde::{Deserialize, Serialize};
use url::Url;

use kdash_protocol::Percent;

#[derive(Debug, Deserialize, Serialize, Clone)]
pub struct Config {
    pub name: String,
    pub image: ImageConfiguration,
    pub battery: BatteryConfig,
    pub time: TimeConfig,
    pub device: DeviceConfig,
}

impl From<Config> for kdash_protocol::Config {
    fn from(value: Config) -> Self {
        Self {
            name: value.name,
            battery: value.battery.into(),
            time: value.time.into(),
            device: value.device.into(),
        }
    }
}

#[derive(Debug, Deserialize, Serialize, Clone)]
pub struct ImageConfiguration {
    pub url: Url,
}

#[derive(Debug, Deserialize, Serialize, Clone)]
pub struct BatteryConfig {
    pub alert: Percent,
    pub low: Percent,
    pub restart_powerd_threshold: Percent,
}

impl From<BatteryConfig> for kdash_protocol::BatteryConfig {
    fn from(value: BatteryConfig) -> Self {
        Self {
            alert: value.alert,
            low: value.low,
            restart_powerd_threshold: value.restart_powerd_threshold,
        }
    }
}

#[serde_with::serde_as]
#[derive(Debug, Deserialize, Serialize, Clone)]
pub struct TimeConfig {
    pub timezone: Tz,

    pub update_schedule: Schedule,

    #[serde_as(as = "serde_with::DurationSeconds<i64>")]
    pub delay_on_error: Duration,

    #[serde_as(as = "serde_with::DurationSeconds<i64>")]
    pub delay_before_suspend: Duration,

    #[serde_as(as = "serde_with::DurationSeconds<i64>")]
    pub delay_low_battery: Duration,
}

impl From<TimeConfig> for kdash_protocol::TimeConfig {
    fn from(value: TimeConfig) -> Self {
        Self {
            timezone: value.timezone,
            update_schedule: value.update_schedule,
            delay_on_error: value.delay_on_error,
            delay_before_suspend: value.delay_before_suspend,
            delay_low_battery: value.delay_low_battery,
        }
    }
}

#[derive(Debug, Deserialize, Serialize, Clone)]
pub struct DeviceConfig {
    pub use_rtc: bool,
    pub full_refresh: bool,
    pub orientation: kdash_protocol::Orientation,
    pub status_box_location: kdash_protocol::Corner,
}

impl From<DeviceConfig> for kdash_protocol::DeviceConfig {
    fn from(value: DeviceConfig) -> Self {
        Self {
            use_rtc: value.use_rtc,
            full_refresh: value.full_refresh,
            orientation: value.orientation,
            status_box_location: value.status_box_location,
        }
    }
}
